local L = LibStub("AceLocale-3.0"):GetLocale("JeypLoathebHealSorter")

JeypLoathebHealSorter.commPrefix = "JeypLoathebHS";
JeypLoathebHealSorter.sPlayerName = UnitName("player")
JeypLoathebHealSorter.sPlayerGUID = UnitGUID("player")
JeypLoathebHealSorter.sActiveSpellName = nil
JeypLoathebHealSorter.sCooldownSpellName = nil
JeypLoathebHealSorter.sSporeSpellName = nil

local tLoathebSpells = {
	sActiveSpellName = L["Corrupted Mind"], -- Limits Healing casts to 1 / min
	sCooldownSpellName = L["Corrupted Mind"], -- 1 min cooldown
	sSporeSpellName = L["Fungal Bloom"], -- Crit Buff
}
local tTestSpells = {
	sActiveSpellName = L["Power Word: Fortitude"],
	sCooldownSpellName = L["Renew"],
	sSporeSpellName = L["Power Word: Shield"],
}

-- ACE CALLBACKS
function JeypLoathebHealSorter:OnInitialize()
	self:InitOptions()
	self:InitHealerFrames()
	
	self.IconLauncher = LibStub("LibDataBroker-1.1"):NewDataObject("JeypLoathebHealSorter", {
		type = "launcher",
		text = "JeypLoathebHealSorter",
		icon = "Interface/Icons/spell_shadow_auraofdarkness",
		OnClick = function(self, button)
			JeypLoathebHealSorter:IconClick()
		end,
		OnEnter = function(self)
			GameTooltip:SetOwner(self, "ANCHOR_LEFT")
			JeypLoathebHealSorter:DrawTooltip()
		end,
		OnLeave = function(self)
			GameTooltip:Hide()
		end
	})
	
	self.icon = LibStub("LibDBIcon-1.0")
	self.icon:Register("JeypLoathebHealSorter", self.IconLauncher, self.db.global)
	
	self.iGreatestTimestamp = 0
	self.iGreatestTimestampSeparator = 0
	self.sRaidSeparator = ","
	self.fLastTimeInCombat = GetTime()
	self.bReadyForReset = false
	self.tHealerOrder = {}
	
	self:RegisterComm(self.commPrefix);
	SLASH_JEYPLOATHEBHS1 = "/jlhs"
	SLASH_JEYPLOATHEBHS2 = "/jeyploathebhealsorter"
	SlashCmdList["JEYPLOATHEBHS"] = function(msg)
		JeypLoathebHealSorter:IconClick()
	end 
	
	self:SetSpellNames()
	self:UpdateFontSize()
	self:HideAnchor()
end

function JeypLoathebHealSorter:SetSpellNames()
	local tSpellNames = nil
	if self.db.global.bTestMode then 
		tSpellNames = tTestSpells
	else
		tSpellNames = tLoathebSpells
	end
	
	JeypLoathebHealSorter.sActiveSpellName = tSpellNames.sActiveSpellName
	JeypLoathebHealSorter.sCooldownSpellName = tSpellNames.sCooldownSpellName
	JeypLoathebHealSorter.sSporeSpellName = tSpellNames.sSporeSpellName
end

function JeypLoathebHealSorter:OnEnable()
	JeypLoathebHealSorter_Frame:ClearAllPoints()
	JeypLoathebHealSorter_Frame:SetPoint("TOPLEFT", self.db.global.tFramePosition.fX, self.db.global.tFramePosition.fY)
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "SendHealerList")
	self:CheckIfActiveAlreadyPresent()
end

function JeypLoathebHealSorter:CheckIfActiveAlreadyPresent()
	if self:GetUnitAura("player", self.sActiveSpellName) then 
		self:EnableWindow()
		self:SendHealerList()
		self:SendCommMessage(self.commPrefix, "ReqHealerList", "RAID")
	else
		self:DisableWindow(true)
	end
end

function JeypLoathebHealSorter:COMBAT_LOG_EVENT_UNFILTERED() 
	local timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, param12, param13, param14, param15, param16, param17, param18, param19 = CombatLogGetCurrentEventInfo()
	
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESHED" then 
		local spellId, spellName, spellSchool = param12, param13, param14
		if destGUID == self.sPlayerGUID and self.sActiveSpellName == spellName and (self.db.global.bTestMode or destGUID ~= sourceGUID) then -- Both debuffs have the same name in English, but can be differenciated by who casts the debuff. The 2min debugg is cast (and refreshed) by Loatheb. The 1min silence is cast by the player.
			JeypLoathebHealSorter:EnableWindow()
			if event == "SPELL_AURA_APPLIED" then 
				-- First application - Send Healer List
				self:SendHealerList()
				-- Check that everyone is alive
				if self.tHealerList then
					for sPlayer, tPlayerData in pairs(self.tHealerList) do 
						tPlayerData.bIsDead = UnitIsDeadOrGhost(sPlayer)
					end
				end
			end
		elseif self.sCooldownSpellName == spellName and self.tHealerList[destName] ~= nil and (self.db.global.bTestMode or destGUID == sourceGUID) then 
			self.tHealerList[destName].bCanCast = false
			self.tHealerList[destName].fCooldownEnd = GetTime() + 60
			if destName == self.tHealerOrder[1] then 
				self:GoToNextPlayer()
			else
				self:UpdateFrame()
			end
		elseif self.sSporeSpellName == spellName and self.tHealerList[destName] ~= nil then 
			self.tHealerList[destName].bHasCritBuff = true
			self.tHealerList[destName].fCritBuffEnd = GetTime() + 90
		end
	elseif event == "SPELL_AURA_REMOVED" or event == "SPELL_AURA_BROKEN"  or event == "SPELL_AURA_BROKEN_SPELL" then 
		local spellId, spellName, spellSchool = param12, param13, param14
		if destGUID == self.sPlayerGUID and self.sActiveSpellName == spellName and (self.db.global.bTestMode or destGUID ~= sourceGUID) then 
			JeypLoathebHealSorter:DisableWindow()
		elseif self.sCooldownSpellName == spellName and self.tHealerList[destName] ~= nil and (self.db.global.bTestMode or destGUID == sourceGUID) then 
			self.tHealerList[destName].bCanCast = true
			self.tHealerList[destName].fCooldownEnd = 0
			self:UpdateFrame()
		elseif self.sSporeSpellName == spellName and self.tHealerList[destName] ~= nil then 
			self.tHealerList[destName].bHasCritBuff = false
			self.tHealerList[destName].fCritBuffEnd = 0
		end
	elseif event == "UNIT_DIED" then
		if self.tHealerList[destName] ~= nil then 
			self.tHealerList[destName].bIsDead = true
			if destName == self.tHealerOrder[1] then 
				self:GoToNextPlayer()
			else
				self:UpdateFrame()
			end
		end
	elseif event == "SPELL_RESURRECT" then
		if self.tHealerList[destName] ~= nil then 
			self.tHealerList[destName].bIsDead = false
			self:UpdateFrame()
		end
	end
end

function JeypLoathebHealSorter:SendHealerList()
	--print("SendHealerList")
	if self:CanSendData() then 
		self:SendCommMessage(self.commPrefix, "Separator," .. self.db.global.iDataTimestamp .. "," .. self.db.global.sSeparator, "RAID")
		self:SendCommMessage(self.commPrefix, "HealerList," .. self.db.global.iDataTimestamp .. "," .. self.db.global.sHealerList, "RAID")
	end
end
	
function JeypLoathebHealSorter:OnCommReceived(prefix, message, distribution, sender)
	local tMessageData = {}
	for match in string.gmatch(message, "[^,]+") do
		if tMessageData[3] ~= nil then 
			tMessageData[3] = tMessageData[3] .. "," ..match -- Separator can be a comma, so just append everything
		else
			table.insert(tMessageData, match)
		end
	end
	
	--print(prefix, distribution, sender, "======", tMessageData[1], tMessageData[2], tMessageData[3])
	if tMessageData[1] == "ReqHealerList" then 
		self:SendHealerList()
	elseif tMessageData[1] == "HealerList" then 
		self:LoadHealerList(tonumber(tMessageData[2]), tMessageData[3], sender)
		--print("Received List : ", tMessageData[2], tMessageData[3]);
	elseif tMessageData[1] == "Separator" then 
		self:LoadSeparator(tonumber(tMessageData[2]), tMessageData[3], sender)
	else 
		--print(prefix, distribution, sender, "======", tMessageData[1], tMessageData[2])
	end
end

function JeypLoathebHealSorter:EnableWindow()
	if UnitInRaid("player") then 
		JeypLoathebHealSorter_Frame:Show()
		JeypLoathebHealSorter_Frame:SetAlpha(1);
	
		if self.cooldownRefreshTimer == nil then
			self.cooldownRefreshTimer = self:ScheduleRepeatingTimer(function() 
				self:UpdateTimers()		
			end, 0.5)
		end
	elseif self.db.global.bTestMode then
		print("[JeypLoathebHealSorter] Enter Raid Mode to Enable Window")
	end
end

function JeypLoathebHealSorter:DisableWindow(bInstant)
	if self.bAnchorVisible then 
		return
	end
	
	if bInstant then 
		JeypLoathebHealSorter_Frame:Hide()
	else 
		UIFrameFadeOut(JeypLoathebHealSorter_Frame, 1.0, 1.0, 0.0)
	end
	if self.cooldownRefreshTimer then 
		self:CancelTimer(self.cooldownRefreshTimer)
		self.cooldownRefreshTimer = nil
	end
end

function JeypLoathebHealSorter:IconClick()
	if JeypLoathebHealSorter.Dialog.OpenFrames["JeypLoathebHealSorter"] then
		JeypLoathebHealSorter.Dialog:Close("JeypLoathebHealSorter")
	else
		JeypLoathebHealSorter.Dialog:Open("JeypLoathebHealSorter")
		JeypLoathebHealSorter.Dialog.OpenFrames["JeypLoathebHealSorter"]:EnableResize(false)
	end
end

function JeypLoathebHealSorter:DrawTooltip() 
	GameTooltip:ClearLines();
	GameTooltip:AddLine("|cff2256aeJeypLoathebHealSorter|r")
	GameTooltip:Show()
end

function JeypLoathebHealSorter:GoToNextPlayer()
	local sNewPlayer = self:GetNextPlayer(self.tHealerOrder[1], {})
	
	if sNewPlayer == self.tHealerOrder[1] then 
		-- Couldn't find any valid Player Just pick next one!
		self.tHealerOrder[1] = self.tHealerList[self.tHealerOrder[1]].sNextPlayer
	else
		self.tHealerOrder[1] = sNewPlayer
	end
	self:UpdateFrame()	
end

function JeypLoathebHealSorter:GetNextPlayer(sPlayer, tFilteredPlayers)
	if sPlayer == nil or self.tHealerList[sPlayer] == nil then 
		return nil
	end
	
	--print(0, sPlayer)
	local i = 1
	local sNextPlayer = self.tHealerList[sPlayer].sNextPlayer
	local sLowestPlayer = nil
	local bLowestIsNotInRaid = false
	local bLowestIsCD = false
	while (not self.tHealerList[sNextPlayer].bCanCast) or self.tHealerList[sNextPlayer].bIsDead or self.tHealerList[sNextPlayer].bNotInRaid or tFilteredPlayers[sNextPlayer] do 
		--print(i, sNextPlayer)
		if not tFilteredPlayers[sNextPlayer] then 
			if sLowestPlayer == nil then
				sLowestPlayer = sNextPlayer
				--print("    NIL")
				if self.tHealerList[sNextPlayer].fCooldownEnd > 0 then 
					bLowestIsCD = true
				elseif self.tHealerList[sNextPlayer].bNotInRaid then 
					bLowestIsNotInRaid = true
				end
			elseif not (bLowestIsCD or bLowestIsNotInRaid) and self.tHealerList[sNextPlayer].bIsDead then 
				sLowestPlayer = sNextPlayer
				--print("    Dead")
			elseif not bLowestIsNotInRaid and (self.tHealerList[sNextPlayer].fCooldownEnd > 0 and (self.tHealerList[sLowestPlayer].fCooldownEnd > self.tHealerList[sNextPlayer].fCooldownEnd)) then 
				bLowestIsCD = true
				sLowestPlayer = sNextPlayer
				--print("    CD")
			elseif not bLowestIsNotInRaid and self.tHealerList[sNextPlayer].bNotInRaid then
				bLowestIsNotInRaid = true
				sLowestPlayer = sNextPlayer
				--print("    NotInRaid")
			else
				--print("    Nothing")
			end 
		else
				--print("    Filtered")
		end

		if self.tHealerList[sNextPlayer].sNextPlayer == sPlayer then
			--print("             RETURN", sLowestPlayer)
			return sLowestPlayer 
		end
		
		i = i + 1
		sNextPlayer = self.tHealerList[sNextPlayer].sNextPlayer
	end
	
	--print("             FINISH", sNextPlayer)
	return sNextPlayer
end

function JeypLoathebHealSorter:PlayerNameWithClassColor(sName) 
	if sName == nil then return "" end
	
	local sClass = select(2, UnitClass(sName))
	if sClass == nil then 
		return sName
	else 
		return "|c" .. select(4, GetClassColor(sClass)) .. sName .. "|r"
	end
end

function JeypLoathebHealSorter:SPairs(t) 
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end
    
    table.sort(keys)
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function JeypLoathebHealSorter:SplitString(sString, sSeparator)
	local tSubStrings = {}
	for match in string.gmatch(sString, "[^" .. sSeparator .. "]+") do
	  table.insert(tSubStrings, match)
	end
	return tSubStrings
end

function JeypLoathebHealSorter:GetUnitAura(unit, spell, filter)
  for i = 1, 255 do
    local name, _, _, _, _, _, _, _, _, spellId = UnitAura(unit, i, filter)
    if name == nil then return end
    if spell == spellId or spell == name then
      return UnitAura(unit, i, filter)
    end
  end
end