local L = LibStub("AceLocale-3.0"):GetLocale("JeypLoathebHealSorter")
local LSM = LibStub( "LibSharedMedia-3.0" )
JeypLoathebHealSorter.Dialog = LibStub("AceConfigDialog-3.0")

local defaults = {
	global = {
		tFramePosition = {
			fX = 0,
			fY = 0,
		},
		bIgnoreNonParty = false,
		bTestMode = false,
		bSendData = false,
		sHealerList = "Jheyp - Jeyp - Kazolk - Boulette - Zengu - Deshiken",
		iDataTimestamp = 0,
		iHealersToDisplay = 5,
		sSeparator = ",",
		iFontSize = 20,
		sFont = "Arial Narrow",
		sBackground = "None",
		tBackgroundColor = {
			r = 1.0,
			g = 1.0,
			b = 1.0,
			a = 0.0,
		},
		sBorder = "None",
		tBorderColor = {
			r = 1.0,
			g = 1.0,
			b = 1.0,
			a = 0.0,
		},
	},
}

JeypLoathebHealSorter.options = {
	name = "JeypLoathebHealSorter - Options",
	handler = JeypLoathebHealSorter,
	type = 'group',
	args = {
		testMode = {
			type = "toggle",
			name = L["Test Mode"],
			order = 1,
			get = "getTestMode",
			set = "setTestMode",
		},
		ignoreNonParty = {
			type = "toggle",
			name = L["Ignore Healers not in raid"],
			order = 2,
			get = "getIgnoreNonParty",
			set = "setIgnoreNonParty",
		},
		healersToDisplay = {
			type = "range",
			name = L["Healers To Display"],
			order = 3,
			get = "getHealersToDisplay",
			set = "setHealersToDisplay",
			min = 1,
			max = 15,
			softMin = 1,
			softMax = 15,
			step = 1,
			width = "full",
		},
		WindowAppearance = {
		  name = L["Display"],
		  type = "group",
		  inline = true,
		  order = 4,
		  args={
			bg = {
				order = 1,
				name = L["Background"],
				type = "select",
				dialogControl = "LSM30_Background",
				values = LSM:HashTable( LSM.MediaType.BACKGROUND ),
				get = "getBackground",
				set = "setBackground",
				width = 1.,
			},
			bgColor = {
				order = 2,
				name = L["Background Color"],
				type = "color",
				hasAlpha = true,
				get = "getBackgroundColor",
				set = "setBackgroundColor",
				width = 1.,
			},
			border = {
				order = 3,
				name = L["Border"],
				type = "select",
				dialogControl = "LSM30_Border",
				values = LSM:HashTable( LSM.MediaType.BORDER ),
				get = "getBorder",
				set = "setBorder",
				width = 1.,
			},
			borderColor = {
				order = 4,
				name = L["Border Color"],
				type = "color",
				hasAlpha = true,
				get = "getBorderColor",
				set = "setBorderColor",
				width = 1.,
			},
			font = {
				order = 5,
				name = L["Font"],
				type = "select",
				dialogControl = "LSM30_Font",
				values = LSM:HashTable( LSM.MediaType.FONT ),
				get = "getFont",
				set = "setFont",
				width = 1.,
			},
			fontSize = {
				type = "range",
				name = L["Font Size"],
				order = 6,
				get = "getFontSize",
				set = "setFontSize",
				min = 1,
				max = 40,
				softMin = 1,
				softMax = 40,
				step = 0.1,
				width = 1.,
			},
		  },
		},
		spacern = {
			type = "description",
			order = 5,
			name = " ",
			width = "full",
		},
		HealerList = {
		  name = L["Data Spreading - RL/Assistant only"],
		  type = "group",
		  inline = true,
		  order = 6,
		  args={
			sendData = {
				type = "toggle",
				name = L["Send Data To Raid"],
				order = 1,
				get = "getSendData",
				set = "setSendData",
			},
			separator = {
				type = "input",
				name = L["Separator"],
				order = 2,
				get = "getSeparator",
				set = "setSeparator",
			},
			healerList = {
				type = "input",
				name = L["Healer List"],
				order = 3,
				get = "getHealerList",
				set = "setHealerList",
				width = "full",
			},
		  },
		},
		toggleAnchor = {
			type = "execute",
			order = 7,
			name = L["Toggle Anchor"],
			func = "ToggleAnchor",
			width = "full",
		},
	}
}

function JeypLoathebHealSorter:CanSendData() 
	return (IsInRaid("player") and ((UnitIsGroupAssistant("player") or UnitIsGroupLeader("player")) and self.db.global.bSendData))
end

function JeypLoathebHealSorter:setBorderColor(info, r, g, b, a)
	self.db.global.tBorderColor.r = r;
	self.db.global.tBorderColor.g = g;
	self.db.global.tBorderColor.b = b;
	self.db.global.tBorderColor.a = a;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBorderColor(info)
	return self.db.global.tBorderColor.r, self.db.global.tBorderColor.g, self.db.global.tBorderColor.b, self.db.global.tBorderColor.a;
end

function JeypLoathebHealSorter:setSeparator(info, value)
	self.db.global.sSeparator = string.sub(value, 1, 1);
	self.db.global.iDataTimestamp = GetServerTime();
	if self:CanSendData() then
		self:SendHealerList()
	else
		self:ReLoadHealerList()
	end
end

function JeypLoathebHealSorter:getSeparator(info)
	return self.db.global.sSeparator;
end

function JeypLoathebHealSorter:setBorder(info, value)
	self.db.global.sBorder = value;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBorder(info)
	return self.db.global.sBorder;
end


function JeypLoathebHealSorter:setBackgroundColor(info, r, g, b, a)
	self.db.global.tBackgroundColor.r = r;
	self.db.global.tBackgroundColor.g = g;
	self.db.global.tBackgroundColor.b = b;
	self.db.global.tBackgroundColor.a = a;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBackgroundColor(info)
	return self.db.global.tBackgroundColor.r, self.db.global.tBackgroundColor.g, self.db.global.tBackgroundColor.b, self.db.global.tBackgroundColor.a;
end

function JeypLoathebHealSorter:setBackground(info, value)
	self.db.global.sBackground = value;
	self:ReloadBackdrop()
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getBackground(info)
	return self.db.global.sBackground;
end

function JeypLoathebHealSorter:setFont(info, value)
	self.db.global.sFont = value;
	self:UpdateFonts()
end

function JeypLoathebHealSorter:getFont(info)
	return self.db.global.sFont;
end

function JeypLoathebHealSorter:setFontSize(info, value)
	self.db.global.iFontSize = value;
	self.fMargin = value * 0.5
	self:UpdateFontSize()
end

function JeypLoathebHealSorter:getFontSize(info)
	return self.db.global.iFontSize;
end

function JeypLoathebHealSorter:setHealerList(info, value)
	self.db.global.sHealerList = value;
	self.db.global.iDataTimestamp = GetServerTime();
	if self:CanSendData() then
		self:SendHealerList()
	end
end

function JeypLoathebHealSorter:getHealerList(info)
	return self.db.global.sHealerList;
end

function JeypLoathebHealSorter:setSendData(info, value)
	self.db.global.bSendData = value;
	if self:CanSendData() then
		self:SendHealerList()
	end
end

function JeypLoathebHealSorter:getSendData(info)
	return self.db.global.bSendData;
end

function JeypLoathebHealSorter:setIgnoreNonParty(info, value)
	self.db.global.bIgnoreNonParty = value;
	self:ReLoadHealerList()
end

function JeypLoathebHealSorter:getIgnoreNonParty(info)
	return self.db.global.bIgnoreNonParty;
end

function JeypLoathebHealSorter:setTestMode(info, value)
	self.db.global.bTestMode = value;
	self:SetSpellNames()
	self:DisableWindow(true)
	self:CheckIfActiveAlreadyPresent()
end

function JeypLoathebHealSorter:getTestMode(info)
	return self.db.global.bTestMode;
end

function JeypLoathebHealSorter:setHealersToDisplay(info, value)
	local iOldValue = self.db.global.iHealersToDisplay
	self.db.global.iHealersToDisplay = value;
	self:ReloadFrame(iOldValue, value)
end

function JeypLoathebHealSorter:getHealersToDisplay(info)
	return self.db.global.iHealersToDisplay;
end

function JeypLoathebHealSorter:resetFramePosition(info)
	self.Frame:ClearAllPoints()
	self.Frame:SetPoint("CENTER", 0, 0)
end

function JeypLoathebHealSorter:InitOptions() 
	self.db = LibStub("AceDB-3.0"):New("JeypLoathebHealSorterOptions", defaults, "Default");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("JeypLoathebHealSorter", JeypLoathebHealSorter.options);
	self.JeypLoathebHealSorterOptions = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("JeypLoathebHealSorter", "JeypLoathebHealSorter");
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("JeypLoathebHealSorter", 420, 550)
end